pyinstaller -n hello hello.py
dir dist\hello
dist\hello\hello.exe
if errorlevel 1 exit 1

pyinstaller -n multiprocessing_test multiprocessing_test.py
dir dist\multiprocessing_test
dist\multiprocessing_test\multiprocessing_test.exe
if errorlevel 1 exit 1




pyinstaller --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyi-archive_viewer --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyi-bindepend --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyi-makespec --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyi-grab_version --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyi-set_version --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
